@@ -22,6 +22,7 @@ from watson_machine_learning_client.functions import Functions
 from watson_machine_learning_client.wml_client_error import NoWMLCredentialsProvided
 from watson_machine_learning_client.wml_client_error import WMLClientError
 import os
+import sys
 
 '''
 .. module:: WatsonMachineLearningAPIClient
@@ -67,6 +68,13 @@ class WatsonMachineLearningAPIClient:
               "Refer to the documentation at 'https://dataplatform.cloud.ibm.com/docs/content/wsj/analyze-data/wml-ai.html'"
               " for the migration process to be able to access new features")
 
+        if (3 == sys.version_info.major) and (6 == sys.version_info.minor):
+            print(
+                "DEPRECATED!! Python 3.6 framework is deprecated and will be removed on Jan 20th, 2021. It will be "
+                "read-only mode starting Nov 20th, 2020. i.e you won't be able to create new assets using this client. "
+                "For cloud, switch to using new client https://pypi.org/project/ibm-watson-machine-learning with "
+                "python 3.7")
+
     def _get_headers(self, content_type='application/json', no_content_type=False):
         headers = {
             'Authorization': 'Bearer ' + self.service_instance._get_token(),
