@@ -13,7 +13,7 @@ from watson_machine_learning_client.utils import INSTANCE_DETAILS_TYPE, FUNCTION
 from watson_machine_learning_client.metanames import FunctionMetaNames
 import os
 import json
-from watson_machine_learning_client.wml_client_error import WMLClientError, UnexpectedType
+from watson_machine_learning_client.wml_client_error import WMLClientError, UnexpectedType, ApiRequestFailure
 from watson_machine_learning_client.wml_resource import WMLResource
 
 _DEFAULT_LIST_LENGTH = 50
@@ -28,11 +28,10 @@ class Functions(WMLResource):
 
     def __init__(self, client):
         WMLResource.__init__(self, __name__, client)
-        Functions._validate_type(client.service_instance.details, u'instance_details', dict, True)
-        Functions._validate_type_of_details(client.service_instance.details, INSTANCE_DETAILS_TYPE)
-
-    def _validate_if_functions_are_available(self, op_name):
-        self._validate_plan(op_name, ['enterpriseAIS', 'ais-enterprise'])
+        if not client.ICP:
+            Functions._validate_type(client.service_instance.details, u'instance_details', dict, True)
+            Functions._validate_type_of_details(client.service_instance.details, INSTANCE_DETAILS_TYPE)
+        self._ICP = client.ICP
 
     @docstring_parameter({'str_type': STR_TYPE_NAME})
     def store(self, function, meta_props):
@@ -48,15 +47,15 @@ class Functions(WMLResource):
 
                 >>> client.functions.ConfigurationMetaNames.get()
 
-            :type meta_props: dict/{str_type}
+            :type meta_props: dict/str
 
             :param function: path to file with archived function content or function (as described above)
-            :type function: {str_type} or function
+            :type function: str or function
 
             :returns: stored function details
             :rtype: dict
 
-            The most simple use is (using `score` function):
+            **Example**:
 
                 >>> def score(payload):
                         values = [[row[0]*row[1]] for row in payload['values']]
@@ -86,7 +85,7 @@ class Functions(WMLResource):
                 >>> }
                 >>> stored_function_details = client.functions.store(score, metadata)
             """
-        self._validate_if_functions_are_available('store function')
+
         function = str_type_conv(function)
         import types
         Functions._validate_type(function, u'function', [STR_TYPE, types.FunctionType], True)
@@ -174,22 +173,49 @@ score = {}()
 
         try:
             if self.ConfigurationMetaNames.RUNTIME_UID not in meta_props:
-                print('No RUNTIME_UID passed. Creating default runtime... ', end="")
-                meta = {
-                    self._client.runtimes.ConfigurationMetaNames.NAME: meta_props[self.ConfigurationMetaNames.NAME] + "_py_3.5",
-                    self._client.runtimes.ConfigurationMetaNames.PLATFORM: {
-                        "name": "python",
-                        "version": "3.5"
+                import sys
+                # print('No RUNTIME_UID passed. Creating default runtime... ', end="")
+                # meta = {
+                #     self._client.runtimes.ConfigurationMetaNames.NAME: meta_props[self.ConfigurationMetaNames.NAME] + "_py_3.5",
+                #     self._client.runtimes.ConfigurationMetaNames.PLATFORM: {
+                #         "name": "python",
+                #         "version": float(sys.version.split()[0][0:3])
+                #     }
+                # }
+                # runtime_details = self._client.runtimes.store(meta)
+                # runtime_uid = self._client.runtimes.get_uid(runtime_details)
+                version = float(sys.version.split()[0][0:3])
+                if version == 3.6:
+                    runtime_uid="ai-function_0.1-py3.6"
+                else:
+                    runtime_uid ="ai-function_0.1-py3"
+                if not self._ICP:
+                    check_runtime = requests.get(self._href_definitions.get_runtime_href(runtime_uid),headers=self._client._get_headers())
+                else:
+                    check_runtime = requests.get(self._href_definitions.get_runtime_href(runtime_uid),headers=self._client._get_headers(), verify=False)
+                if check_runtime.status_code != 200:
+                    print('No matching default runtime found. Creating one...', end="")
+                    meta = {
+                        self._client.runtimes.ConfigurationMetaNames.NAME: meta_props[
+                                                                               self.ConfigurationMetaNames.NAME] + "-"+str(version),
+                        self._client.runtimes.ConfigurationMetaNames.PLATFORM: {
+                            "name": "python",
+                            "version": str(version)
+                        }
                     }
-                }
-                runtime_details = self._client.runtimes.store(meta)
-                runtime_uid = self._client.runtimes.get_uid(runtime_details)
-                print('SUCCESS\n\nSuccessfully created default runtime with uid: {}'.format(runtime_uid))
+                    runtime_details = self._client.runtimes.store(meta)
+                    runtime_uid = self._client.runtimes.get_uid(runtime_details)
+                    print('SUCCESS\n\nSuccessfully created runtime with uid: {}'.format(runtime_uid))
+                else:
+                    print('Using default runtime with uid: {}'.format(runtime_uid))
                 meta_props[self.ConfigurationMetaNames.RUNTIME_UID] = runtime_uid
 
             function_metadata = self.ConfigurationMetaNames._generate_resource_metadata(meta_props, client=self._client)
 
-            response_post = requests.post(self._href_definitions.get_functions_href(), json=function_metadata, headers=self._client._get_headers())
+            if not self._ICP:
+                response_post = requests.post(self._href_definitions.get_functions_href(), json=function_metadata, headers=self._client._get_headers())
+            else:
+                response_post = requests.post(self._href_definitions.get_functions_href(), json=function_metadata, headers=self._client._get_headers(), verify=False)
 
             details = self._handle_response(201, u'saving function', response_post)
 
@@ -197,7 +223,10 @@ score = {}()
 
             put_header = self._client._get_headers(no_content_type=True)
             with open(content_path, 'rb') as data:
-                response_definition_put = requests.put(function_content_url, data=data, headers=put_header)
+                if not self._ICP:
+                    response_definition_put = requests.put(function_content_url, data=data, headers=put_header)
+                else:
+                    response_definition_put = requests.put(function_content_url, data=data, headers=put_header, verify=False)
 
         except Exception as e:
             raise e
@@ -217,13 +246,20 @@ score = {}()
         Updates existing function metadata.
 
         :param function_uid: UID of function which define what should be updated
-        :type function_uid: {str_type}
+        :type function_uid: str
 
         :param changes: elements which should be changed, where keys are ConfigurationMetaNames
         :type changes: dict
 
         :return: metadata of updated function
         :rtype: dict
+
+        **Example**
+         >>> metadata = {
+         >>> client.functions.ConfigurationMetaNames.NAME:"updated_function"
+         >>> }
+         >>> function_details = client.functions.update(function_uid, changes=metadata)
+
         """
         function_uid = str_type_conv(function_uid)
         self._validate_type(function_uid, u'function_uid', STR_TYPE, True)
@@ -238,7 +274,10 @@ score = {}()
         url = self._href_definitions.get_function_href(function_uid)
         headers = self._client._get_headers()
         headers['If-Match'] = details['entity']['function_revision']['url'].split('/')[-1]
-        response = requests.patch(url, json=patch_payload, headers=headers)
+        if not self._ICP:
+            response = requests.patch(url, json=patch_payload, headers=headers)
+        else:
+            response = requests.patch(url, json=patch_payload, headers=headers, verify=False)
         updated_details = self._handle_response(200, u'function patch', response)
 
         return updated_details
@@ -249,15 +288,21 @@ score = {}()
             Download function content from repository to local file.
 
             :param function_uid: stored function UID
-            :type function_uid: {str_type}
+            :type function_uid: str
 
             :param filename: name of local file to create (optional)
-            :type filename: {str_type}
+            :type filename: str
+
+            :returns: path to the downloaded file
+            :rtype: str
+
+            .. note::
+               If filename is not specified, the default filename is "downloaded_function.gz".
 
             Side effect:
                 save function to file.
 
-            A way you might use me is:
+            **Example**
 
             >>> client.functions.download(function_uid, 'my_func.tar.gz')
         """
@@ -273,9 +318,12 @@ score = {}()
         artifact_content_url = self._href_definitions.get_function_latest_revision_content_href(artifact_uid)
 
         try:
-            r = requests.get(artifact_content_url, headers=self._client._get_headers(), stream=True)
-
-            self._handle_response(200, 'downloading function content', r, False)
+            if not self._ICP:
+                r = requests.get(artifact_content_url, headers=self._client._get_headers(), stream=True)
+            else:
+                r = requests.get(artifact_content_url, headers=self._client._get_headers(), stream=True, verify=False)
+            if r.status_code != 200:
+                raise ApiRequestFailure(u'Failure during {}.'.format("downloading function"), r)
 
             downloaded_model = r.content
             self._logger.info(u'Successfully downloaded artifact with artifact_url: {}'.format(artifact_url))
@@ -287,8 +335,8 @@ score = {}()
         try:
             with open(filename, 'wb') as f:
                 f.write(downloaded_model)
-            self._logger.info(u'Successfully saved artifact to file: \'{}\''.format(filename))
-            return None
+            print(u'Successfully saved artifact to file: \'{}\''.format(filename))
+            return os.getcwd()+"/"+filename
         except IOError as e:
             raise WMLClientError(u'Saving function content with artifact_url: \'{}\' failed.'.format(filename), e)
 
@@ -298,9 +346,12 @@ score = {}()
             Delete function from repository.
 
             :param function_uid: stored function UID
-            :type function_uid: {str_type}
+            :type function_uid: str
 
-            A way you might use me is:
+            :returns: returns the status message ("SUCCESS" or FAILED")
+            :rtype: str
+
+            **Example**
 
             >>> client.functions.delete(function_uid)
         """
@@ -312,14 +363,16 @@ score = {}()
         for deployment_details in deployments_details:
             deployment_uid = self._client.deployments.get_uid(deployment_details)
             print('Deleting orphaned function deployment \'{}\'... '.format(deployment_uid), end="")
-            self._client.deployments.delete(deployment_uid)
-            print('SUCCESS')
+            dep_delete_status = self._client.deployments.delete(deployment_uid)
+            print(dep_delete_status)
 
         function_endpoint = self._href_definitions.get_function_href(function_uid)
         self._logger.debug(u'Deletion artifact function endpoint: {}'.format(function_endpoint))
-        response_delete = requests.delete(function_endpoint, headers=self._client._get_headers())
-
-        self._handle_response(204, u'function deletion', response_delete, False)
+        if not self._ICP:
+            response_delete = requests.delete(function_endpoint, headers=self._client._get_headers())
+        else:
+            response_delete = requests.delete(function_endpoint, headers=self._client._get_headers(), verify=False)
+        return self._handle_response(204, u'function deletion', response_delete, False)
 
     @docstring_parameter({'str_type': STR_TYPE_NAME})
     def get_details(self, function_uid=None, limit=None):
@@ -327,15 +380,15 @@ score = {}()
             Get metadata of function. If no function uid is specified all functions metadata is returned.
 
             :param function_uid: stored function UID (optional)
-            :type function_uid: {str_type}
+            :type function_uid: str
 
             :param limit: limit number of fetched records (optional)
             :type limit: int
 
             :returns: stored function(s) metadata
-            :rtype: dict
+            :rtype: dict (if uid is not None) or {"resources": [dict]} (if uid is None)
 
-            A way you might use me is:
+            **Example**
 
             >>> function_details = client.functions.get_details(function_uid)
             >>> function_details = client.functions.get_details()
@@ -358,9 +411,9 @@ score = {}()
             :type function_details: dict
 
             :returns: uid of stored function
-            :rtype: {str_type}
+            :rtype: str
 
-            A way you might use me is:
+            **Example**
 
             >>> function_uid = client.functions.get_uid(function_details)
         """
@@ -380,9 +433,9 @@ score = {}()
             :type function_details: dict
 
             :returns: url of stored function
-            :rtype: {str_type}
+            :rtype: str
 
-            A way you might use me is:
+            **Example**
 
             >>> function_url = client.functions.get_url(function_details)
         """
@@ -398,11 +451,17 @@ score = {}()
             :param limit: limit number of fetched records (optional)
             :type limit: int
 
-            A way you might use me is
+           :returns: None
+           :rtype: None
+
+           .. note::
+               This function only prints the list of stored functions
+
+            **Example**
 
             >>> client.functions.list()
         """
-        self._validate_if_functions_are_available('list functions')
+
         function_resources = self.get_details()[u'resources']
         function_values = [(m[u'metadata'][u'guid'], m[u'entity'][u'name'], m[u'metadata'][u'created_at'],
                               m[u'entity'][u'type']) for m in function_resources]
